$TITLE        SAM UPDATE PROGRAM

*-------------------------------------------------
*Contacts
*Fousseini Traore: Fousseini.Traore@cgiar.org
*Alhassane Camara: Alhassane.Camara@USherbrooke.ca
*we thank Dominique van der mensbrugghe for helpful suggestions  
*-------------------------------------------------

*  Available methods:
*     ME          Maximum Entropy
*     LSQ         Least squares
*     MLSQ        Modified least squares
*     Hellinger   Hellinger method
*     Huber       Huber method
*     MHuber      Modified Huber

acronyms ME, LSQ, MLSQ, Hellinger, Huber, MHuber ;

scalar method  / LSQ / ;

*=========== 1.Input names of matrix accounts ==================================
SETS
   act         "Activities" /
               agr-a    "Agriculture"
               lff-a    "Livestock, fisheries and forestry"
               xtr-a    "Extraction"
               fdp-a    "Food processing"
               xmn-a    "Other industry"
               psv-a    "Private services"
               pub-a    "Public administration"
               /
   comm        "Commodities" /
               agr-c    "Agriculture"
               lff-c    "Livestock, fisheries and forestry"
               xtr-c    "Extraction"
               fdp-c    "Food processing"
               xmn-c    "Other industry"
               psv-c    "Private services"
               pub-c    "Public administration"
               /
   fac         "Factors of production" /
               lab      "Labor"
               cap      "Capital"
               /
   inst        "Institutions" /
               trc      "Domestic margins?"
               hhd      "Households"
               dtax     "Direct taxes"
               mtax     "Import taxes"
               stax     "Sales taxes"
               gov      "Government"
               i_s      "Investment/savings"
               row      "Rest of the world"
               /
   i           "All accounts" /
               set.act
               set.comm
               set.fac
               set.inst
               "Tot"    "Total"
               /

   INT(I)      "All accounts except Total" /
               set.act
               set.comm
               set.fac
               set.inst
               /
   INTS(INT)   "All accounts except gov, s-i, Total" /
               set.act
               set.comm
               set.fac
               trc      "Domestic margins?"
               hhd      "Households"
               dtax     "Direct taxes"
               mtax     "Import taxes"
               stax     "Sales taxes"
*              gov      "Government"
*              i_s      "Investment/savings"
               row      "Rest of the world"
               /
;
ALIAS(I,J) ; ALIAS(INT,JNT) ; ALIAS(INTS,JNTS) ;

*&&& Updating the SAM - Define the subsets
SETS
   ICOM(INT)      "Commodities"            / set.comm /
   ISEC(INT)      "Institutions"           / set.inst /
   IROW(ISEC)     "Rest of the world"      / row /
   IDO(ISEC)      "Domestic"               / hhd, gov /
   ICOMINV(ICOM)  "Investment commodities" / xmn-c /
   ICOMOTH(ICOM)  "Non-investment commodities" /
                     agr-c    "Agriculture"
                     lff-c    "Livestock, fisheries and forestry"
                     xtr-c    "Extraction"
                     fdp-c    "Food processing"
*                    xmn-c    "Other industry"
                     psv-c    "Private services"
                     pub-c    "Public administration"
                  /
   IACT(INT)      "Activities"             / set.act /
   asec           "Aggregate activities" /
                     Agr      "Agriculture"
                     Ind      "Industry"
                     Srv      "Services"
                  /
   asrv(asec)     "Aggregate service activity" / srv /
   mapa(asec,iact)   /
      agr.(agr-a, lff-a)
      ind.(xtr-a, fdp-a, xmn-a)
      srv.(psv-a, pub-a)
   /
   IFAC(INT)      "Factors"                / set.fac /
   imrg(INT)      "Margin?"                / trc /
   iinv(INT)      "Investment account"     / i_s /
   igov(INT)      "Government account"     / gov /
;

*============ 2.Input initial matrix ===========================================
PARAMETER
   SAM0(I,J)      "Initial matrix"

$CALL GDXXRW SAM0.xls Par=SAM0 rng=B4:AA29
$GDXIN SAM0.gdx
$LOAD SAM0

*==================================================================================================================================================

   SAM4(I,J)      "Initial matrix divided by sum of all entries"
   TOT4           "Sum of all entries (less total)"
;

TOT4          = SUM((INT,JNT),SAM0(INT,JNT)) ;
SAM4(INT,JNT) = SAM0(INT,JNT)/TOT4 ;
DISPLAY SAM0, SAM4 ;

*===============================================================================================================================================
*============ 4.Matrix of negative values defined and negative values transposed
PARAMETER
   NEG(I,J)       "Matrix of negative values"
;

NEG(INT,JNT)$(SAM0(INT,JNT) LT 0)  = 1 ;
SAM0(INT,JNT)$(SAM0(JNT,INT) LT 0) = SAM0(INT,JNT)-SAM0(JNT,INT) ;
SAM0(JNT,INT)$(SAM0(JNT,INT) LT 0) = 0 ;

*============ 3.Initial matrix values transformed in proportion ================
PARAMETER
   TOTO           "Initial matrix total"
   GDPO           "Initial GDP at purchasing price"

   VAB0(asec)     "Initial value added by major activity"

   CHGO           "Initial total consumption including final consumption by governement"
   GFCO           "Initial investment"
*  CAO            "Initial current account balance"
*  SGO            "Initial GOVERNMENT current account"
;

TOTO          = SUM((INT,JNT), SAM0(INT,JNT)) ;
SAM0(INT,JNT) = SAM0(INT,JNT)/TOTO ;

*  GDP at market price -- sum over all commodities and institutions less imports
GDPO          = SUM[(ICOM,ISEC), SAM0(ICOM,ISEC)]-SUM[(IROW,ICOM), SAM0(IROW,ICOM)] ;
VAB0(asec)    = sum((ifac,iact)$mapa(asec,iact), SAM0(IFAC, IACT))
              + sum((imrg,icom), sam0(imrg,icom))$asrv(asec) ;

*  Private and public expenditures
CHGO          = SUM[(ICOM,IDO),SAM0(ICOM,IDO)] + SUM[(imrg, ICOMOTH),SAM0(imrg,ICOMOTH)] ;
*  Investment
GFCO          = SUM[(ICOM,IINV),SAM0(ICOM,IINV)] + SUM[(imrg,ICOMINV),SAM0(imrg,ICOMINV)] ;
*CAO          = -sum((iinv,irow), SAM0(iinv, irow)) ;
*SGO          =  sum((iinv,igov), SAM0(iinv, igov)) ;

*=========== 5.Non-zero logs parameter =========================================
SCALARS
   delta          "Non-zero logs parameter"
;
delta = 0.000000000000001 ;

*==============================gowth============================================

PARAMETERS
   gwt(asec) / agr 0.780, ind 1.362, srv 1.061 /
   gwt_chg   / 0.984 /
   gwt_gfc   / 1.337 /
   savg_tgt  / 0.0704 /
;

*==========================CHECKS======================================================

PARAMETERS
   indi(asec)
   indi4
   indi5
;
indi(asec) = VAB0(asec) / GDPO ;

indi4 = CHGO/GDPO ;

indi5 = GFCO/GDPO ;

DISPLAY  indi, indi4, indi5 ;

*=========== 6.Variable definition =============================================
VARIABLES
   SAM(I,J)       "New SAM with transposed negative values"
   OPT            "Distance variable"
   GDP            "Gross Domestic Product at purchasing price"
   VAB(asec)      "Value added by broad activity"

   CHG            "Total consumption including final consumption by governement"
   GFC            "FBC"
   SG             "Public sold"
;

*=========== 7.Equation definition and initialization ==========================
EQUATIONS
   OPTIMEQ        "Optimization criterion"
   EQUAL(INT)     "Equality between matrix and row sums"
   UNITEQ         "Proportions sum equal one"
   GDPEQ          "GDP purchasing price"
   CONSEQ1        "Ratio Investment to GDP"
   CONSEQ2        "Ratio Public Final Consumption Expenditure to GDP"
   CONSEQ3(asec)  "Value Added Share by broad activity"
   CONSEQ6(asec)  "Value Added Share growth by broad activity"
   CONSEQ9        "Ratio investment to GDP growth"
   CONSEQ10       "Ratio final consumption to GDP"
   CONSEQ11       "Public saving"
   CONSEQ12       "Ratio of public sold to GDP"
;


*Entropy optimization criterion

OPTIMEQ..
   OPT =e= (SUM((INT,JNT), SAM(INT,JNT)
        *                 (LOG(SAM(INT,JNT)+delta) - LOG(SAM0(INT,JNT)+delta))))
        $(method = ME)

        +  (SUM((INT,JNT)$(SAM0(INT,JNT) NE 0), sqr[SAM(INT,JNT)-SAM0(INT,JNT)]))
        $(method = LSQ)

        +  (SUM((INT,JNT)$(SAM0(INT,JNT)NE 0), sqr[SAM(INT,JNT)/SAM0(INT,JNT)-1]))
        $(method = MLSQ)

        +  (SUM((INT,JNT)$(SAM0(INT,JNT) NE 0), SQR(SQRT(SAM(INT,JNT))-SQRT(SAM0(INT,JNT)))))
        $(method = Hellinger)

        +  (SUM((INT,JNT)$(SAM0(INT,JNT) NE 0), (2*(.00713315)**2)
        *                  (SQRT(1+SQR((SAM0(INT,JNT)-SAM(INT,JNT))/.00713315))-1)))
        $(method = Huber)

        +  (SUM((INT,JNT)$(SAM0(INT,JNT) NE 0), (2*(.00713315)**2)
        *                  (SQRT(1+SQR((SAM(INT,JNT)/SAM0(INT,JNT)-1)/.00713315))-1)))
        $(method = MHuber)

*       +  (SUM((INT,JNT)$(SAM0(INT,JNT) NE 0),  (2*(.00659389)**2)
*       *                  (SQRT(1+SQR(((-SAM0(INT,JNT) + SAM(INT,JNT))/SAM0(INT,JNT))/.00659389))-1)))
*       $(method = NA)
        ;

*Equality between row and column sums
EQUAL(INT)..
   SUM(JNT,SAM(INT,JNT)) =E= SUM(JNT,SAM(JNT,INT)) ;

*Proportions sum equal one
UNITEQ..
   SUM((INT,JNT),SAM(INT,JNT)) =E= 1 ;

*~~~Assign the GDP variable
GDPEQ..
   GDP =E= SUM[(ICOM,ISEC),SAM(ICOM,ISEC)] - SUM[(IROW,ICOM),SAM(IROW,ICOM)] ;

*~~~ Investment to GDP
CONSEQ1..
   GFC =E= SUM[(ICOM,iinv),SAM(ICOM,iinv)] + SUM[(imrg,ICOMINV),SAM(imrg,ICOMINV)] ;

* final consumption to GDP
CONSEQ2..
   CHG =E= SUM[(ICOM,IDO),SAM(ICOM,IDO)] + SUM[(imrg,ICOMOTH),SAM(imrg,ICOMOTH)] ;

* Value added by broad activity
CONSEQ3(asec)..
   VAB(asec) =E= SUM[(IFAC,IACT)$mapa(asec,iact), SAM(IFAC,IACT)]
              +   sum((imrg,icom), sam(imrg,icom))$asrv(asec) ;

*===============================================growth=========================================

*Value added share growth
CONSEQ6(asec)..
   (VAB(asec)/GDP) =E= gwt(asec)*(VAB0(asec)/GDPO) ;

*~~~ratio  Investment to GDP
CONSEQ9..
   (GFC/GDP) =E= gwt_gfc*(GFCO/GDPO) ;

* ratio final consumption to GDP
CONSEQ10..
   (CHG/GDP) =E= gwt_chg*(CHGO/GDPO) ;

CONSEQ11..
   SG =E= sum((iinv,igov), SAM(iinv,igov)) ;

CONSEQ12..
   (SG/GDP) =E= savg_tgt ;

*========= 8.Variable initialization============================================

*Cell values between 0 and infinity
SAM.L(I,J)       = SAM0(I,J) ;
SAM.LO(INT,JNT)  = 0 ;
SAM.UP(INT,JNT)  = +INF ;

*SAM.FX(INT,JNT)$(NOT SAM0(INT,JNT))  = 0 ;
***with restriction
*activate $ontext and $offtext if no restriction
*$ontext

SAM.FX(INTS,JNTS)$(NOT SAM0(INTS,JNTS))      = 0 ;
loop(igov,
   SAM.FX(INTS,igov)$(NOT SAM0(INTS,igov)) = 0 ;
   SAM.FX(igov,JNTS)$(NOT SAM0(igov,JNTS)) = 0 ;
   SAM.FX(igov,igov)$(NOT SAM0(igov,igov)) = 0 ;
) ;

loop(iinv,
   SAM.FX(INTS,iinv)$(NOT SAM0(INTS,iinv)) = 0 ;
   SAM.FX(iinv,JNTS)$(NOT SAM0(iinv,JNTS)) = 0 ;
   SAM.FX(iinv,iinv)$(NOT SAM0(iinv,iinv)) = 0 ;
   loop(igov,
      SAM.FX(igov,iinv)$(NOT SAM0(igov,iinv)) = 0 ;
   ) ;
) ;
*$offtext

OPT.L = 0 ;
GDP.L = GDPO ;

VAB.L(asec) = VAB0(asec) ;
CHG.L       = CHGO ;
GFC.L       = GFCO ;
SG.L        = 0 ;

*========== 9.Model solving=====================================================
MODEL SAMBALANCE / ALL / ;
SAMBALANCE.workspace = 10;
*SAMBALANCE.optfile = 1;
OPTION NLP          = CONOPT3 ;
OPTION iterlim      = 99999;
SOLVE SAMBALANCE USING NLP MINIMIZING OPT;
*$exit

*============ 10.Results copied into new matrix=================================
PARAMETER
   NSAM(I,J)   "New (balanced) matrix"
   NSAM1(I,J)
;
NSAM(INT,JNT) = SAM.L(INT,JNT) ;

*Negative values retransposed to original position
NSAM(INT,JNT)$(NEG(INT,JNT)=1) = -NSAM(JNT,INT) ;
NSAM(JNT,INT)$(NEG(INT,JNT)=1) = 0 ;

*Transformation in SAM transaction flow

NSAM1(INT,JNT) = NSAM(INT,JNT) ;

***new SAM flows
NSAM(INT,JNT) = NSAM(INT,JNT) * TOTO ;
*$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

*==========================================ratios and indicators===================
PARAMETERS
   ind(asec)   "Valued added share"
   ind4        "Ratio Final consumption to GDP"
   ind5        "Ratio FBC to GDP"
;

ind(asec) = VAB.L(asec)/GDP.L ;

ind4 = CHG.L/GDP.L ;

ind5 = GFC.L/GDP.L ;

display ind, ind4, ind5 ;
*$exit

*=============================================================================
PARAMETER
   SAM1(I,J)   "Current matrix (Observed SAM)"
   Diff1(I,J)  "Difference between oberved SAM and simulated SAM (proportions)"
   Diff2(I,J)  "Difference between observed SAM and simulated SAM (percentage change)"
   TOT1        "Total of SAM1"
   SAM2        "SAM1 in proportions"
   FACT(I,J)   "Parameter for value added computation (factors and acivities)"
   FI(I,J)     "Parameter for share of factors in Value added computation"
   VA(J)       "Value added"
   CI(J)       "Intermediate consumption"
   SHVAB(J)    "Share of value added in production"
*  SAM2(I,J)
;

SET
   AFF(I,J)    "Most affected accounts"
;

* Observed SAM
$CALL GDXXRW SAM1.xls Par=SAM1 rng=B4:AA29
$GDXIN SAM1.gdx
$LOAD SAM1

TOT1           = SUM((INT,JNT),SAM1(INT,JNT)) ;
SAM2(INT,JNT)  = SAM1(INT,JNT)/TOT1 ;

Diff1(INT,JNT) = SAM2(INT,JNT)-NSAM1(INT,JNT) ;
Diff2(INT,JNT)$NSAM1(INT,JNT) = (SAM2(INT,JNT)/NSAM1(INT,JNT)-1) ;

*account with a difference in % greater than 75%.
AFF(INT,JNT)$(ABS[Diff2(INT,JNT) GT 0.75]) = YES ;

FACT(IFAC,IACT) = NSAM1(IFAC,IACT) ;
VA(IACT)        = SUM(IFAC,FACT(IFAC,IACT)) ;

FI(IFAC,IACT)  = FACT(IFAC,IACT)/VA(IACT) ;
CI(IACT)       = SUM(ICOM,NSAM1(ICOM,IACT)) ;
SHVAB(IACT)    = VA(IACT) /(VA(IACT)+CI(IACT)) ;

display  AFF, NSAM1;

parameter checktotalrow(int), checktotalcol(jnt) ;
checktotalrow(int) = sum(jnt, NSAM(INT,JNT)) ;
checktotalcol(jnt) = sum(int, NSAM(INT,JNT)) ;
display checktotalrow,checktotalcol ;

*$exit


*---Output final SAM------------------------------------------------------------

Execute_Unload "NSAM1.gdx"
Execute "GDXXRW NSAM1.gdx par=NSAM1 rng=A1"

Execute_Unload "Diff1.gdx"
Execute "GDXXRW Diff1.gdx par=Diff1 rng=A1"

Execute_Unload "Diff2.gdx"
Execute "GDXXRW Diff2.gdx par=Diff2 rng=A1"

Execute_Unload "FI.gdx"
Execute "GDXXRW FI.gdx par=FI rng=A1"

Execute_Unload "SHVAB.gdx"
Execute "GDXXRW SHVAB.gdx par=SHVAB rng=A1"

Execute_Unload "SAM2.gdx"
Execute "GDXXRW SAM2.gdx par=SAM2 rng=A1"

Execute_Unload "SAM4.gdx"
Execute "GDXXRW SAM4.gdx par=SAM4 rng=A1"
*===============================================================================
DISPLAY AFF;
